# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

from thx.core.models import UserModel, DateModel, StatusModel, OrderedModel, CleanModel


class PaymentForm(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        unique=True,
        max_length=50,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True,
        verbose_name=_("description"),
    )
    fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("fee"),
    )
    discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("discount"),
    )

    class Meta:
        verbose_name = _('payment form')
        verbose_name_plural = _('payment forms')
        ordering = ('ordering', 'code',)
        permissions = (
            ("list_paymentform", "Can list payment form"),
            ("detail_paymentform", "Can detail payment form"),
            ("enable_paymentform", "Can enable payment form"),
            ("disable_paymentform", "Can disable payment form"),
        )

    def __str__(self):
        return f"{self.code} - {self.name}"


class PaymentInstallment(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    payment_form = models.ForeignKey(
        PaymentForm,
        related_name='installments',
        on_delete=models.PROTECT,
        verbose_name=_("payment form"),
    )
    code = models.CharField(
        unique=True,
        max_length=255,
        blank=True,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=511,
        blank=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True,
        verbose_name=_("description"),
    )
    amount = models.IntegerField(
        default=100,
        verbose_name=_("amount %"),
    )
    days = models.IntegerField(
        default=0,
        verbose_name=_("days"),
    )
    end_month = models.BooleanField(
        verbose_name=_("end month"),
    )
    # Additional days beyond the end of the month. It will be used only
    # if `end_month` is equal to True and will be added to the calculated
    # expiration date.
    days_end_month = models.IntegerField(
        default=0,
        verbose_name=_("days after end month"),
    )
    discount = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0.00,
        verbose_name=_("discount %"),
    )
    fee = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0.00,
        verbose_name=_("fee %"),
    )

    class Meta:
        verbose_name = _('payment installment')
        verbose_name_plural = _('payment installments')
        ordering = ('ordering', 'code',)
        permissions = (
            ("list_paymentinstallment", "Can list payment installment"),
            ("detail_paymentinstallment", "Can detail payment installment"),
            ("enable_paymentinstallment", "Can enable payment installment"),
            ("disable_paymentinstallment", "Can disable payment installment"),
        )

    def __str__(self):
        return f"{self.code} - {self.name}"

    def set_code(self):
        if not self.code:
            self.code = "{}_{}_{}".format(
                self.payment_form.code, self.amount, self.days
            )

    def set_name(self):
        if not self.name:
            self.name = "Codice Pagamento: {} {}% {} gg.".format(
                self.payment_form.code, self.amount, self.days
            )

    def save(self, **kw):
        self.set_code()
        self.set_name()
        super(PaymentInstallment, self).save(**kw)

    def get_full_name(self):
        return "{} {}".format(self.payment_form.name, self.days)

    full_name = property(get_full_name)
